'' =================================================================================================
''
''   File....... jm_sht11_logger.spin 
''   Purpose....  
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 21 OCT 2015 
''
'' =================================================================================================

{

   Commands:

   temp c            - return temperature in Celsius
   temp f            - return temperature in Fahrenheit
   humidity          - display temp-compenstate relative humidity
   thnow             - display current time, temperature (C), and humidity
   clock             - display current time
   clock hh mm       - set clock hours and minutes (seconds set to 0)
   clock hh mm ss    - set clock hours, minutes, and seconds
   logfile           - display current log file name
   logfile filename  - set log file name
   samples           - display logging samples
   samples nn        - set logging samples
   logging           - display logging status, Yes or No
   logging yes       - start logging (if samples > 0)
   logging no        - stop logging
   files             - display files on sd card
   view filename     - display contents of filename (if text)

   Note: For filename input, quotes should be used around name to allow any character, e.g.,

     VIEW "RH151022.CSV"

   When logging is initiated samples will be taken on the new minute until the number of
   samples is decremented to 0. Samples are written to the current log file. After all
   samples are taken, the log file is closed. The format is CSV which can be opened and
   graphed using a spreadsheet (e.g., Excel or OpenOffice Calc)

}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { pab io pins }                                               
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                      
                                                    
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                      
                                                      
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                             
                                                      
  LED_27  = 27                                                   ' leds (on audio outputs)
  LED_26  = 26                                             
                                                      
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23
  SD_DO   = 22

  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19                               
  ADC_DI  = 18                                
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16
  SVO_4   = 15
  SVO_3   = 14
  SVO_2   = 13
  SVO_1   = 12

  ' app connections

  DIO     = 15                                                   ' SHT11 module connections
  SCK     = 14 

  RX2     = 11                                                   ' to rn-42 tx pin (3.3v mode)
  TX2     = 10                                                   ' to rn-42 rx pin


con 

  #(-1), IS_ON, IS_OFF

  #0, NO, YES
  
  #0, USB, XBEE                                                  ' serial connection modes

  COM_MODE = USB                                                 ' use programming port 

  BUF_SIZE = 80


obj

' main                                                           ' * master spin cog
  time   : "jm_time"                                             '   delays/timing
  prng   : "jm_prng"                                             '   random #s
  io     : "jm_io"                                               '   basic io
  parser : "jm_parser"                                           '   text parsing engine
  str    : "jm_strings"                                          '   string manipulations
  sht    : "jm_sht11"                                            '   SHT11 sensor interface
  adc    : "jm_adc124s021"                                       '   for onboard adc
  rtc    : "jm_simple_rtc"                                       ' * 24-hr clock
  term   : "jm_fullduplexserial"                                 ' * serial for terminal
  sd     : "fsrw"                                                ' * uSD file IO
                                                                  
' * uses cog when loaded
  

var

  long  logging                                                  ' true or false
  long  samples                                                  ' samples left

  byte  workbuf[BUF_SIZE]


dat

  logfile       byte    "thlog000.csv", 0                        ' default log file

  header        byte    "Time, TempC, Humidity", 0               ' header for CSV output
  
  csvfield      byte    ", n.n", 0[11]                           ' numeric csv field

  terror        byte    "Error: Temperature", 0
  herror        byte    "Error: Humidity", 0
  therror       byte    ", -1000, -1000", 0                      ' extreme values for sensor problem  
  

pub main | c, check, now, last, tled

  setup

  last := -1

  repeat
    c := term.rxcheck
    if (c => 0)
      check := parser.enqueue(c)                                 ' send to parser                      
      if (check)                                                 ' have valid tokens?                  
        process_cmd                                                                                  
        parser.reset                                             ' reset parser for next command       

    if ((logging == true) and (samples > 0))
      rtc.read_all(@now)
      if (now.byte[1] == 0)                                      ' seconds = 0
        if (now.byte[2] <> last)                                 ' new minutes
          last := now.byte[2]                                    ' mark this one
          io.high(LED_26)                                        ' blip sample led
          tled := time.millis                                    ' start led timer
          time_tmp_humi(@workbuf, BUF_SIZE)                      ' create time+temp+humidity report   
          wr_line(@workbuf, YES, YES)                            ' write to file
          if (--samples == 0)                                    ' decrement samples, if done
            logging := No                                        '  kill logging
            close_file                                           '  close log file
            io.low(LED_27)                                       '  kill visual indicator

      if (outa[LED_26] and ((time.millis - tled) > 100))         ' kill sample led       
        io.low(LED_26)


pub setup

  time.start                                                     ' setup for timing/delays

  prng.seed(cnt, cnt ~> 2, $EA7_BEEF, cnt << 2, cnt)             ' setup randomizer                             

  io.start(0, 0)                                                 ' clear pins (this cog)

  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, false)       ' setup parsing engine

  sht.start(SCK, DIO)                                            ' connect to SHT11 module

' adc.start(ADC_CS, ADC_CLK, ADC_DI, ADC_DO)                     ' connect to adc 

  rtc.start                                                      ' start basic rtc

  if (COM_MODE == USB)
    term.start(RX1, TX1, %0000, 115_200)                         ' start serial for usb coms *
  else { xbee }
    term.start(RX2, TX2, %0000, 115_200)                         ' start serial for xbee *
                                                                  
  mount_sd                                                       ' start sd access *


con

  { ------------------------------- }
  {  C O M M A N D   P A R S I N G  } 
  { ------------------------------- }
  

dat { parser characters and tokens }

  CHAR_SET      byte    "$%+-_"
                byte    ".0123456789"
                byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0   
                 
  TOKEN_LIST    byte    "TEMP", 0                                ' display temperature
                byte    "C", 0                                   ' type qualifiers for TEMP
                byte    "F", 0 
                byte    "HUMIDITY", 0                            ' display humidity
                byte    "THNOW", 0                               ' time + temp c + humidity
                byte    "CLOCK", 0                               ' view or set clock    
                byte    "FILES", 0                               ' list directory
                byte    "LOGFILE", 0                             ' show or set log file
                byte    "SAMPLES", 0                             ' show or set samples
                byte    "LOGGING", 0                             ' show or set logging status
                byte    "YES", 0
                byte    "NO", 0
                byte    "VIEW", 0                                ' display contents of specified file
                
                        
con { enumerated token indexes }

   #0, T_TEMP, T_C, T_F, T_HUMI, T_THNOW
   #5, T_CLOCK  
   #6, T_FILES
   #7, T_LOGFILE
   #8, T_SAMPLES
   #9, T_LOGGING, T_YES, T_NO
  #12, T_VIEW
  
  TOKEN_COUNT = T_VIEW + 1                                       ' total # of known tokens


pub process_cmd | tidx

  tidx := parser.get_token_id(parser.token_addr(0))              ' get index of command token
  
  case tidx
    T_TEMP    : cmd_temp
    T_HUMI    : cmd_humi
    T_THNOW   : cmd_thnow
    T_CLOCK   : cmd_clock    
    T_FILES   : cmd_files
    T_LOGFILE : cmd_logfile
    T_SAMPLES : cmd_samples
    T_LOGGING : cmd_logging
    T_VIEW    : cmd_view
    
    
pub cmd_temp | tidx, tmp, tmode

'' Read and display current temperature

  if (parser.token_count <> 2)                                   ' correct count?                                  
    return                                                       '  no, abort

  tidx := parser.get_token_id(parser.token_addr(1))              ' get index of mode

  if ((tidx <> T_C) and (tidx <> T_F))                           ' validate mode
    return

  tmp := sht.temperature                                         ' measure temperature
  if ((tmp < -40_00) or (tmp > 120_00))                          ' if out-of-range
    term.str(@terror)                                            '  report sensor error
    term.tx(term#CR)
    return

  ' reading is okay

  if (tidx == T_C)
    tmp := (sht.temp_c + 5) / 10                                 ' round up to 10ths
    tmode := "C"
  else
    tmp := (sht.temp_f + 5) / 10   
    tmode := "F"

  term.dec(tmp / 10)                                             ' whole degrees     
  term.tx(".")                    
  term.tx("0" + (tmp // 10))                                     ' tenths   
  term.tx(tmode)                  
  term.tx(term#CR)


pub cmd_humi | tc, rh

'' Read and display current humidity

  if (parser.token_count <> 1)                                   ' correct count?                                  
    return                                                       '  no, abort

  tc := sht.temperature                                          ' measure temperature
  if ((tc < -40_00) or (tc > 120_00))                            ' if out-of-range
    term.str(@terror)                                            '  report sensor error
    term.tx(term#CR)
    return

  rh := sht.humidity                                             ' measure humidity
  if ((rh < 0) or (rh > 100_00))                                 ' if out-of-range
    term.str(@herror)                                            '  report sensor error
    term.tx(term#CR)
    return

  rh := (sht.rh_true + 5) / 10                                   ' get compensated humidity
  term.dec(rh / 10)            
  term.tx(".")                    
  term.tx("0" + (rh // 10))     
  term.tx("%")                  
  term.tx(term#CR)


pub cmd_thnow 

'' Display time + temp + humidity string

  if (parser.token_count <> 1)            
    return
 
  time_tmp_humi(@workbuf, BUF_SIZE)                              ' create CSV report strin 

  term.str(@workbuf)                                             ' write to terminal
  term.tx(term#CR)


pub time_tmp_humi(p_str, size) | tc, rh, value

'' Build time + tempC + humidity string

  bytefill(p_str, 0, size)                                       ' clear buffer space
  str.concat(p_str, rtc.p_hhmm, p_str)                           ' start with time
  
  tc := sht.temperature                                          ' measure temperature
  if ((tc < -40_00) or (tc > 120_00))                            ' if out-of-range
    str.concat(p_str, @therror, p_str)                           ' error message 
    return

  rh := sht.humidity                                             ' measure humidity
  if ((rh < 0) or (rh > 100_00))                                 ' if out-of-range
    str.concat(p_str, @therror, p_str)                           ' error message   
    return

  value := (sht.temp_c + 5) / 10                                 ' round up to 10ths
  tenths(value, @csvfield[2])                                    ' convert to string
  str.concat(p_str, @csvfield, p_str)

  value := (sht.rh_true + 5) / 10                                ' round up to 10ths
  tenths(value, @csvfield[2])                                    ' convert to string
  str.concat(p_str, @csvfield, p_str) 


pub tenths(value, p_str) | td, div

'' Convert value (in 0.1 units) to string
'' -- ensure p_str large enough to accomodate value
''    * e.g., negx requires 13 bytes

  td := ||(value // 10)                                          ' save tenths digit
  value /= 10                                                    ' remove tenths from value

  if (value < 0)                                                 ' if negative
    byte[p_str++] := "-"                                         '  write sign
    ||value                                                      '  convert to positive

  div := 1_000_000_000                                           ' start with max divisor

  repeat
    if ((value / div) == 0)                                      ' skip leading zeros
      div /= 10                                                  ' reduced divisor
    else
      quit

  repeat while (div)                                             ' convert whole part
    byte[p_str++] := "0" + (value / div)                         ' digit to ASCII
    value //= div                                                ' remove last digit
    div /= 10                                                    ' update divisor

  byte[p_str++] := "."
  byte[p_str++] := "0" + td                                      ' tenths
  byte[p_str] := 0                                               ' terminate


pub cmd_clock

'' Review or set clock
'' -- command is overloaded
'' -- 0 parameters = display current clock
'' -- 2 parameters = set hours and minutes registers
'' -- 3 parameters = set hours, minutes, and seconds registers
'' -- all parameters qualified before any changes

  if (parser.token_count == 1)                                   ' no parameters                               
    term.str(rtc.p_hhmmss)                                       '  yes, show current time
    term.tx(term#CR)
    return

  if (parser.token_count == 3)                                   ' 2 parameters = set hours & mins
    if (parser.token_in_range(1, 0, 23))    
      if (parser.token_in_range(2, 0, 59))
        rtc.set(parser.token_value(1), parser.token_value(2), 0, 0) 
    return

  if (parser.token_count == 4)                                   ' 3 parameters = set hours, mins, & secs
    if (parser.token_in_range(1, 0, 23))
      if (parser.token_in_range(2, 0, 59))  
        if (parser.token_in_range(3, 0, 59))
          rtc.set(parser.token_value(1), parser.token_value(2), parser.token_value(3), 0)    
    return
    

pub cmd_files

'' Directory of files on system SD card

  if (parser.token_count <> 1)                                   ' correct count?                                  
    return                                                       '  no, abort

  term.tx(term#CR)
  directory
  repeat 2
    term.tx(term#CR)


pub cmd_logfile | p, len, d

'' Show logfile or set new one

  if (parser.token_count == 1)                                   ' no parameters                               
    term.str(@logfile)                                           '  yes, show current filename
    term.tx(term#CR)
    return

  bytefill(@workbuf, 0, BUF_SIZE)                                ' clear working buffer
  parser.copy_token(1, @workbuf)                                 ' copy new filename to buffer
  str.trim(@workbuf)                                             ' remove extranious spaces

  p := str.chcount(".", @workbuf)                                ' count "." in workbuf

  if (p == 0)
    workbuf[8] := 0                                              ' truncate to 8 chars
    str.concat(@workbuf, string(".CSV"), @workbuf)               ' add default extension

  elseif (p == 1)
    if ((len => 5) and (len =< 12))                              ' validate
      d := str.first(".", @workbuf)                              ' position of .
      len := strsize(@workbuf)                                   ' length of new file name
      p := len - d                                               ' length of extension + .
      if ((d == 0) or (p > 4) or (p < 2))
        return
    else
      return

  else
    return

  bytemove(@logfile, @workbuf, len+1)                            ' update logfile

  term.str(@logfile)                                             ' show it
  term.tx(term#CR)
  
    
pub cmd_samples

  if (parser.token_count == 1)                                   ' no parameters                               
    term.dec(samples)
    term.tx(term#CR)
    return

  if (parser.token_count <> 2)
    return

  ifnot (parser.token_is_number(1))
    return

  samples := 0 #> parser.token_value(1) <# 1440    

 
pub cmd_logging | tidx

  if (parser.token_count == 1)                                   ' no parameters                               
    if (logging)
      term.str(string("Yes", 13))
    else
      term.str(string("No", 13))
    return

  if (parser.token_count <> 2)                                   ' must be 1 or 2
    return

  tidx := parser.get_token_id(parser.token_addr(1))              ' convert parameter to index  

  if (tidx == T_YES)
    if (samples > 0)
      logging := true                                            ' set flag
      open_file(@logfile, "w")                                   ' open log file
      wr_line(@header, YES, YES)                                 ' write column header to file 
      io.high(LED_27)                                            ' visual indicator on

  elseif (tidx == T_NO)
    logging := false                                             ' clear flag
    close_file                                                   ' close the file
    io.low(LED_27)                                               ' visual indicator off


pub cmd_view | check, len

'' View contents of selected logfile

  logging := false                                               ' kill logging process
  samples := 0
  close_file

  bytefill(@filename, 0, 13)

  if (parser.token_count == 2)                                   ' name specified?
    parser.copy_token(1, @filename)                              '  yes, copy to filename
  else
    return  

  ifnot (open_file(@filename, "r"))                              ' if file won't open
    term.tx(term#CR)                                             '  report error
    term.str(string("File not found"))
    term.tx(term#CR)
    return
    

  ' file found -- show contents

  term.tx(term#CR)
  term.str(string("File: "))
  term.str(@filename)
  repeat 2
    term.tx(term#CR)

  show_file(@filename)                                           ' display contents

  repeat 2
    term.tx(term#CR)
  

con

  { ----------------- }
  {  F I L E   I / O  } 
  { ----------------- }


var

  long  mounted

  byte  filename[8 + 1 + 3 + 1]                                  ' 8.3 plus 0
  byte  linebuf[80 + 1]                                          ' line buffer
  

pub mount_sd | check

'' Attempt to mount SD card
'' -- returns true if SD card successfully mounted

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  mounted := (check == 0)   

  return mounted


pub dismount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub open_file(p_str, mode) | check

'' Attempt to open file
'' -- p_str is pointer to filename (z-string)
'' -- mode is [lower-case] single character, (e.g., "r" for read)
'' -- sd card must be mounted before open

  if (mounted)
    check := \sd.popen(p_str, mode)                              ' attempt to open
    if (check == 0)                                               
      if (mode == "r")                     
        \sd.seek(0)                                              ' force sector read 
      return true
    else
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub rd_line(p_str, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- p_str is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                       ' index into string
                                                                  
  repeat n                                                        
    c := \sd.pgetc                                               ' get a character
    if (c < 0)                                                   ' end of file
      if (len == 0)                                              ' if nothing
        return -1                                                ' return empty
      else                                                        
        quit                                                      
                                                                  
    if (c <> 10)                                                 ' if not LF char
      byte[p_str++] := c                                         '  add to buffer
      len++                                                       
    else                                                         ' if LF
      quit                                                       '  we're done
                                                                  
  byte[p_str] := 0                                               ' terminate end of line
                                                                  
  return len
  

pub wr_line(p_str, usecr, uself)

'' Writes z-string to open text file
'' -- set usecr to true to add CR
'' -- set uself to true to add LF

  \sd.pputs(p_str)
  if (usecr)
    \sd.pputc(13)
  if (uself)
    \sd.pputc(10)


pub rd_buf(p_buf, n) | check

'' Reads n bytes from file to buffer at p_buf
'' -- returns number of bytes read

  return \sd.pread(p_buf, n)


pub wr_buf(p_buf, n) | check   

'' Writes n bytes to file from buffer at p_buf
'' -- returns number of bytes written

  return \sd.pwrite(p_buf, n)


pub directory | fcount, check, idx, p_fname, fsize

  sd.opendir                                                     ' setup for .nextfile
  fcount := 0                                                    ' reset files count
                                                                  
  term.str(string("Directory", term#CR, term#CR))                 
                                                                  
  repeat                                                          
    bytefill(@filename, 0, 13)                                    
    check := \sd.nextfile(@filename)                             ' get next file in root
    if (check == 0)                                              ' if found
      fcount += 1                                                '  increment file count
      term.str(@filename)                                        '  show name
      term.tx(term#CR)                                            
    else                                                         ' else
      quit                                                       '  done
                                                                  
  if (fcount > 0)
    term.tx(term#CR)
    term.dec(fcount)
    term.str(string(" file(s) found."))
  else
    term.str(string("No files found."))

  sd.pclose


pub show_file(p_file) | len

  ifnot (mounted)
    term.str(string("SD not mounted.", 13))
    return

  if (open_file(p_file, "r"))
    repeat
      bytefill(@linebuf, 0, 81)
      len := rd_line(@linebuf, 80)
      if (len > 0)
        term.str(@linebuf)
        time.pause(5)
      else
        close_file
        quit 
    

dat { license }

{{

  Copyright (c) 2015 Jon McPhalen  

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}