'' =================================================================================================
''
''   File....... jm_simple_rtc.spin
''   Purpose.... Simple time-only RTC
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 20 OCT 2015
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' i2c / eeprom
  SCL = 28


con

  #0, R_HUNS, R_SECS, R_MINS, R_HRS


var

  long  cog
  long  stack[64]

  long  busy

  byte  reg[4]                                                   ' rtc registers


pub start

'' Start RTC with time set to midnight

  startx(0, 0, 0)


pub startx(h, m, s)

'' Start RTC at specific time

  stop                                                           ' stop if already running

  set(h, m, s, 0)                                                ' preset time

  cog := cognew(rtc, @stack) + 1                                 ' start rtc cog


pub stop

'' Stops RTC cog

  if (cog)
    cogstop(cog - 1)
    cog := 0

  resume                                                         ' clear hold flag  
  reset                                                          ' clear rtc regs
  

pub reset

'' Reset RTC to midnight

  repeat while busy

  bytefill(@reg, 0, 4)
  

pub set(h, m, s, c)

'' Set RTC registers

  repeat while busy                                              ' let current update finish

  reg[R_HRS]  := 0 #> h <# 23                                    ' set registers
  reg[R_MINS] := 0 #> m <# 59
  reg[R_SECS] := 0 #> s <# 59
  reg[R_HUNS] := 0 #> c <# 99


pub write(n, value)

'' Write to specified register

  repeat while busy                                              ' let current update finish     

  case n
    R_HRS  : reg[R_HRS]  := 0 #> value <# 23
    R_MINS : reg[R_MINS] := 0 #> value <# 59
    R_SECS : reg[R_SECS] := 0 #> value <# 59
    R_HUNS : reg[R_HUNS] := 0 #> value <# 99
     
  
pub read(n)

'' Read selected register

  if ((n => R_HUNS) and (n =< R_HRS))
    return reg[n]
  else
    return -1


pub read_all(p_dest)

  repeat while (busy)

  bytemove(p_dest, @reg, 4)


pub p_hhmmsscc | now

  read_all(@now)                                                 ' get current time

  Time_HHMMSSCC[ 0] := "0" + (now.byte[3]  / 10)                 ' hours
  Time_HHMMSSCC[ 1] := "0" + (now.byte[3] // 10) 

  Time_HHMMSSCC[ 3] := "0" + (now.byte[2]  / 10)                 ' minutes
  Time_HHMMSSCC[ 4] := "0" + (now.byte[2] // 10)

  Time_HHMMSSCC[ 5] := ":"                                       ' replace (if overwritten)
     
  Time_HHMMSSCC[ 6] := "0" + (now.byte[1]  / 10)                 ' seconds
  Time_HHMMSSCC[ 7] := "0" + (now.byte[1] // 10)

  Time_HHMMSSCC[ 8] := "."                                       

  Time_HHMMSSCC[ 9] := "0" + (now.byte[0]  / 10)                 ' cents
  Time_HHMMSSCC[10] := "0" + (now.byte[0] // 10)

  return @Time_HHMMSSCC 

  
pub p_hhmmss | now

  read_all(@now)                                                 ' get current time

  Time_HHMMSSCC[0] := "0" + (now.byte[3]  / 10)                  ' hours
  Time_HHMMSSCC[1] := "0" + (now.byte[3] // 10) 

  Time_HHMMSSCC[3] := "0" + (now.byte[2]  / 10)                  ' minutes
  Time_HHMMSSCC[4] := "0" + (now.byte[2] // 10)

  Time_HHMMSSCC[5] := ":"                                        ' replace (if overwritten)

  Time_HHMMSSCC[6] := "0" + (now.byte[1]  / 10)                  ' seconds
  Time_HHMMSSCC[7] := "0" + (now.byte[1] // 10)

  Time_HHMMSSCC[8] := 0

  return @Time_HHMMSSCC 


pub p_hhmm | now

  read_all(@now)                                                 ' get current time

  Time_HHMMSSCC[0] := "0" + (now.byte[3]  / 10)                  ' hours
  Time_HHMMSSCC[1] := "0" + (now.byte[3] // 10) 

  Time_HHMMSSCC[3] := "0" + (now.byte[2]  / 10)                  ' minutes
  Time_HHMMSSCC[4] := "0" + (now.byte[2] // 10)

  Time_HHMMSSCC[5] := 0                                          ' replace (if overwritten)

  return @Time_HHMMSSCC 

  
pub p_mins | m

'' Return time in minutes as 4-character string

  m := minutes

  Time_XXXX[0] := "0" + (m  / 1000)
  Time_XXXX[1] := "0" + (m // 1000 / 100)
  Time_XXXX[2] := "0" + (m //  100 / 10)
  Time_XXXX[3] := "0" + (m //   10)

  return @Time_XXXX 


pub minutes

'' Return hours & minutes as consolidated value
'' -- 0 (midnight) to 1439 (11:59PM)

  repeat while (busy)

  return (reg[R_HRS] * 60) + reg[R_MINS]


pub seconds

'' Return current time as seconds

  repeat while (busy)

  return (reg[R_HRS] * 3600) + (reg[R_MINS] * 60) + reg[R_SECS]


pub hold

'' Hold RTC at current time

  busy := true


pub resume

'' Allow RTC to run

  busy := false


pri rtc | ms10, sync

  ms10 := clkfreq / 100 

  sync := cnt
  repeat
    busy := 0
    repeat while (busy)
      sync := cnt
    waitcnt(sync += ms10)
    busy := 1
    if (++reg[R_HUNS] == 100)
      reg[R_HUNS] := 0
      if (++reg[R_SECS] == 60) 
        reg[R_SECS] := 0
        if (++reg[R_MINS] == 60)
          reg[R_MINS] := 0
          if (++reg[R_HRS] == 24)
            reg[R_HRS] := 0           


dat

  Time_HHMMSSCC         byte    "00:00:00.00", 0

  Time_XXXX             byte    "0000", 0
  

dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}